#!/bin/bash

# http://download.osgeo.org/gdal/2.1.3/gdal-2.1.3.tar.gz

# Other Sources:
# http://downloads2.esri.com/Software/FileGDB_API_VS2010_1_4.zip
# http://downloada.erdas.com/software/2013/ECWJP2SDK/erdas-ecwjp2sdk-v5.0-windows.zip
# https://www.lizardtech.com/download/developer/MrSID_DSDK-8.5.0.3422-win32-vs10.zip
# https://www.lizardtech.com/download/developer/MrSID_DSDK-8.5.0.3422-win64-vs10.zip
# http://go.microsoft.com/fwlink/?LinkID=239647&clcid=0x409 => sqlncli-x86.msi
# http://go.microsoft.com/fwlink/?LinkID=239648&clcid=0x409 => sqlncli-x64.msi
# msiexec /i sqlncli.msi ADDLOCAL=ALL APPGUID={0CC618CE-F36A-415E-84B4-FB1BFF6967E1} (with SDK)

set -e

#
# run batch file and source environment changes
#
fetchenv() {
	local IFS
	IFS=
	batch=$1	
	shift
	cmd /c set >$TEMP/srcenv
	cmd /c call `cygpath -w $batch` "$@" \>nul 2\>nul \& set >$TEMP/dstenv
	diff -u $TEMP/srcenv $TEMP/dstenv | sed -f envdiff.sed >$TEMP/diffenv
	. $TEMP/diffenv
	PATH=$PATH:/bin:/usr/bin:/cygdrive/c/windows32/system32:/cygdrive/c/windows
	rm -f $TEMP/srcenv $TEMP/dstenv $TEMP/diffenv
}

ARCH=$1
case "$ARCH" in
x86)
	export OSGEO4W_ROOT="C:\\OSGeo4W"
	BITS=32
	VCARCH=x86
	ECWARCH=win32
	FGDBARCH=
	HINT=gdal32.hint
	;;

x86_64)
	export OSGEO4W_ROOT="C:\\OSGeo4W64"
	BITS=64
	VCARCH=x64
	ECWARCH=x64
	FGDBARCH=64
	HINT=gdal.hint
	;;

*)
	echo "Unsupported arch $ARCH" >&2
	exit 1
	;;
esac

for i in $(seq 9 -1 2); do
	[ -f package-$ARCH.$(( i - 1 )).log ] && mv package-$ARCH.$(( i - 1 )).log package-$ARCH.$i.log 
done
[ -f package-$ARCH.log ] && mv package-$ARCH.log package-$ARCH.1.log 

[ -d log ] || mkdir log
if tty -s; then
	exec > >(tee log/package-$ARCH.log) 2>&1
else
	exec >log/package.log 2>&1
fi

fetchenv $OSGEO4W_ROOT/bin/o4w_env.bat

P=gdal
V=2.1.3
B=1

export EXT_NMAKE_OPT=$PWD/nmake$BITS.opt
export FGDB_SDK=$PWD/../../filegdb
export ECW_SDK=$PWD/../../ecw-5
export MRSID_SDK=$PWD/../../MrSID_DSDK-8.5.0.3422-win$BITS-vs10

R=../../../$ARCH/release/$P
for i in python oracle filegdb ecw mrsid sosi mss; do
	[ -d $R/$P-$i ] || mkdir -p $R/$P-$i
done

export DESTDIR=$PWD/install$BITS
export PY2DESTDIR=$PWD/pyinstall2-$BITS
# export PY3DESTDIR=$PWD/pyinstall3-$BITS

set -x

rm -rf $DESTDIR
rm -rf $PY2DESTDIR
# rm -rf $PY3DESTDIR

mkdir -p $DESTDIR/etc/ini
mkdir -p $DESTDIR/share/gdal

fetchenv /cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 10.0/VC/vcvarsall.bat $VCARCH
PATH=/cygdrive/c/Program\ Files*/Microsoft\ Visual\ Studio\ 10.0/Common7/IDE:$PATH

cp ini.bat $DESTDIR/etc/ini/$P.bat

mkdir -p $PY2DESTDIR/etc/postinstall
cp $P-python.postinstall $PY2DESTDIR/etc/postinstall/$P-python.bat

# mkdir -p $PY3DESTDIR/etc/postinstall
# cp $P-python3.postinstall $PY3DESTDIR/etc/postinstall/$P-python3.bat

cp $HINT $R/setup.hint
cp $P-python.hint $R/$P-python/setup.hint
# cp $P-python3.hint $R/$P-python3/setup.hint
cp $P-oracle.hint $R/$P-oracle/setup.hint
cp $P-filegdb.hint $R/$P-filegdb/setup.hint
cp $P-ecw.hint $R/$P-ecw/setup.hint
cp $P-mrsid.hint $R/$P-mrsid/setup.hint
cp $P-sosi.hint $R/$P-sosi/setup.hint
cp $P-mss.hint $R/$P-mss/setup.hint

cp ../LICENSE.TXT $R/$P-$V-$B.txt
cp ../LICENSE.TXT $R/$P-python/$P-python-$V-$B.txt
cp ../LICENSE.TXT $R/$P-oracle/$P-oracle-$V-$B.txt
cp ../LICENSE.TXT $R/$P-mss/$P-mss-$V-$B.txt
cp $FGDB_SDK/license/userestrictions.txt $R/$P-filegdb/$P-filegdb-$V-$B.txt
cp $ECW_SDK/EULA.rtf $R/$P-ecw/$P-ecw-$V-$B.rtf
cp $MRSID_SDK/LICENSE.pdf $R/$P-mrsid/$P-mrsid-$V-$B.pdf

cd ..

[ -f osgeo4w/patched-plugin-clean ] || patch -p1 <osgeo4w/plugin-clean.diff >osgeo4w/patched-plugin-clean

for i in clean default install devinstall; do
	if ! [ -f osgeo4w/no$i ]; then
		nmake /f makefile.vc \
			EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) \
			GDAL_HOME=$(cygpath -w $DESTDIR) \
			FGDB_SDK=$(cygpath -w $FGDB_SDK) \
			ECWDIR=$(cygpath -w $ECW_SDK) \
			MRSID_SDK=$(cygpath -w $MRSID_SDK) \
			$i
	fi
done

[ -f osgeo4w/nopackage ] && exit

cd swig

nmake /f makefile.vc EXT_NMAKE_OPT=$(cygpath -w $EXT_NMAKE_OPT) python

cd python
mkdir -p $PY2DESTDIR/apps/$P/python
$OSGEO4W_ROOT/bin/python.exe setup.py install --prefix=$(cygpath -w $PY2DESTDIR)/apps/python27 --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-2-$ARCH.log
# call py3_env.bat
# mkdir -p $PY3DESTDIR/apps/$P/python3
# $OSGEO4W_ROOT/bin/python3.exe setup.py install --prefix=$(cygpath -w $PY3DESTDIR)/apps/python36 --single-version-externally-managed --record ..\\..\\osgeo4w\\log\\record-3-$ARCH.log

mkdir -p $PY2DESTDIR/bin
cp scripts/*.py $PY2DESTDIR/bin
# mkdir -p $PY3DESTDIR/apps/Python36/Scripts
# cp scripts/*.py $PY3DESTDIR/apps/Python36/Scripts

cd ../../osgeo4w

cp $FGDB_SDK/bin$FGDBARCH/FileGDBAPI.dll $DESTDIR/bin
cp $ECW_SDK/bin/vc100/$ECWARCH/NCSEcw.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk_cdll.dll $DESTDIR/bin
cp $MRSID_SDK/Raster_DSDK/lib/lti_dsdk.dll $DESTDIR/bin
cp $MRSID_SDK/Lidar_DSDK/lib/lti_lidar_dsdk.dll $DESTDIR/bin

tar -C pyinstall2-$BITS --exclude="*.pyc" -cjvf $R/$P-python/$P-python-$V-$B.tar.bz2 apps/Python27 bin etc
# tar -C pyinstall3-$BITS --exclude="*.pyc" --exclude __pycache__ -cjvf $R/$P-python3/$P-python3-$V-$B.tar.bz2 apps/Python36 etc
tar -C install$BITS --remove-files -cjvf $R/$P-filegdb/$P-filegdb-$V-$B.tar.bz2 bin/gdalplugins/ogr_FileGDB.dll bin/FileGDBAPI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-sosi/$P-sosi-$V-$B.tar.bz2 bin/gdalplugins/ogr_SOSI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-oracle/$P-oracle-$V-$B.tar.bz2 bin/gdalplugins/gdal_GEOR.dll bin/gdalplugins/ogr_OCI.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mss/$P-mss-$V-$B.tar.bz2 bin/gdalplugins/ogr_MSSQLSpatial.dll
tar -C install$BITS --remove-files -cjvf $R/$P-ecw/$P-ecw-$V-$B.tar.bz2 bin/gdalplugins/gdal_ECW_JP2ECW.dll bin/NCSEcw.dll
tar -C install$BITS --remove-files -cjvf $R/$P-mrsid/$P-mrsid-$V-$B.tar.bz2 bin/gdalplugins/gdal_MG4Lidar.dll bin/gdalplugins/gdal_MrSID.dll bin/lti_dsdk_cdll.dll bin/lti_lidar_dsdk.dll bin/lti_dsdk.dll
tar -C install$BITS --remove-files -cjvf $R/$P-$V-$B.tar.bz2 bin etc include lib share
tar -C .. -cjvf $R/$P-$V-$B-src.tar.bz2 \
	osgeo4w/$HINT \
	osgeo4w/gdal-ecw.hint \
	osgeo4w/gdal-filegdb.hint \
	osgeo4w/gdal-mrsid.hint \
	osgeo4w/gdal-oracle.hint \
	osgeo4w/gdal-python.hint \
	osgeo4w/gdal-python3.hint \
	osgeo4w/gdal-sosi.hint \
	osgeo4w/gdal-mss.hint \
	osgeo4w/gdal-python.postinstall \
	osgeo4w/ini.bat \
	osgeo4w/plugin-clean.diff \
	osgeo4w/nmake$BITS.opt \
	osgeo4w/package.sh \
	osgeo4w/envdiff.sed
